@@ -1,7 +1,7 @@
 from __future__ import annotations
 import asyncio
 from asyncio import Task
-from enum import Enum, IntFlag
+from enum import Enum
 import inspect
 import logging
 import json
@@ -14,7 +14,7 @@ from dataclasses_json import Undefined, config, DataClassJsonMixin
 from aiohttp_sse_client.client import MessageEvent
 
 from .const import Events
-from .common import HomeConnectError
+from .common import ConditionalLogger, HomeConnectError, GlobalStatus
 from .callback_registery import CallbackRegistry
 from .appliance import Appliance
 from .auth import AuthManager
@@ -29,16 +29,6 @@ class HomeConnect(DataClassJsonMixin):
     """ The main class that wraps the whole data model,
     coordinates the loading of data from the cloud service and listens for update events
     """
-    class HomeConnectStatus(IntFlag):
-        """ Enum for the current status of the Home Connect data loading process """
-        INIT = 0
-        LOADING = 1
-        LOADED = 3
-        UPDATES = 4
-        NOUPDATES = ~4
-        READY = 7
-        LOADING_FAILED = 8
-
 
     class RefreshMode(Enum):
         """ Enum for the supported data refresh modes """
@@ -47,17 +37,16 @@ class HomeConnect(DataClassJsonMixin):
         DYNAMIC_ONLY = 2
         ALL = 3
 
-
     # This is a class variable used as configuration for the dataclass_json
     dataclass_json_config:ClassVar[config] = config(undefined=Undefined.EXCLUDE)
 
     # The data calss fields
     appliances:dict[str, Appliance] = field(default_factory=dict)
-    status:HomeConnect.HomeConnectStatus = \
-        field(
-            default=HomeConnectStatus.INIT,
-            metadata=config(encoder = lambda val: val.name, exclude = lambda val: True)
-        )
+    # status:HomeConnect.HomeConnectStatus = \
+    #     field(
+    #         default=HomeConnectStatus.INIT,
+    #         metadata=config(encoder = lambda val: val.name, exclude = lambda val: True)
+    #     )
 
 
     # Internal fields - not serialized to JSON
@@ -92,11 +81,12 @@ class HomeConnect(DataClassJsonMixin):
         if json_data:
             try:
                 hc = HomeConnect.from_json(json_data)
-                hc.status = cls.HomeConnectStatus.INIT
+                #hc.status = cls.HomeConnectStatus.INIT
                 # manually initialize the appliances because they were created from json
                 for appliance in hc.appliances.values():
                     appliance._homeconnect = hc
                     appliance._callbacks = hc._callbacks
+                    appliance._api = api
             except Exception as ex:
                 _LOGGER.exception("Exception when loading HomeConnect data from JSON", exc_info=ex)
         if not hc:
@@ -113,7 +103,6 @@ class HomeConnect(DataClassJsonMixin):
 
         return hc
 
-
     def start_load_data_task(self,
         refresh:RefreshMode = None,
         on_complete:Callable[[HomeConnect], None] = None,
@@ -137,7 +126,9 @@ class HomeConnect(DataClassJsonMixin):
         on_error:Callable[[HomeConnect, Exception], None] = None
     ) -> None:
         """ Loads or just refreshes the data model from the cloud service """
-        self.status |= self.HomeConnectStatus.LOADING
+        #self.status |= self.HomeConnectStatus.LOADING
+        GlobalStatus.set_status(GlobalStatus.Status.RUNNING)
+        GlobalStatus.unset_status(GlobalStatus.Status.LOADING_FAILED)
 
         try:
             if refresh == self.RefreshMode.NOTHING:
@@ -167,6 +158,7 @@ class HomeConnect(DataClassJsonMixin):
                                 appliance = await Appliance.async_create(self, ha)
                                 self.appliances[haid] = appliance
                             await self._callbacks.async_broadcast_event(self.appliances[ha['haId']], Events.PAIRED)
+                            await self._callbacks.async_broadcast_event(self.appliances[ha['haId']], Events.DATA_CHANGED)
                             _LOGGER.debug("Loadded appliance: %s", self.appliances[ha['haId']].name)
                         elif haid in self.appliances:
                             _LOGGER.warning("The appliance (%s) is disconnected when loading for the first time", haid)
@@ -178,10 +170,12 @@ class HomeConnect(DataClassJsonMixin):
                         await self._callbacks.async_broadcast_event(self.appliances[haId], Events.DEPAIRED)
                         del self.appliances[haId]
 
-            self.status |= self.HomeConnectStatus.LOADED
+            #self.status |= self.HomeConnectStatus.LOADED
+            GlobalStatus.set_status(GlobalStatus.Status.LOADED)
         except Exception as ex:
             _LOGGER.warning("Failed to load data from Home Connect (%s)", str(ex), exc_info=ex)
-            self.status = self.HomeConnectStatus.LOADING_FAILED
+            #self.status = self.HomeConnectStatus.LOADING_FAILED
+            GlobalStatus.set_status(GlobalStatus.Status.LOADING_FAILED)
             if on_error:
                 if inspect.iscoroutinefunction(on_error):
                     await on_error(self, ex)
@@ -252,7 +246,8 @@ class HomeConnect(DataClassJsonMixin):
                 _LOGGER.debug("Connecting to SSE stream")
                 event_source = await self._api.async_get_event_stream('/api/homeappliances/events')
                 await event_source.connect()
-                self.status |= self.HomeConnectStatus.UPDATES
+                #self.status |= self.HomeConnectStatus.UPDATES
+                GlobalStatus.set_status(GlobalStatus.Status.UPDATES)
 
                 async for event in event_source:
                     _LOGGER.debug("Received event from SSE stream: %s", str(event))
@@ -264,10 +259,12 @@ class HomeConnect(DataClassJsonMixin):
             except asyncio.CancelledError:
                 break
             except ConnectionRefusedError as ex:
-                self.status &= self.HomeConnectStatus.NOUPDATES
+                #self.status &= self.HomeConnectStatus.NOUPDATES
+                GlobalStatus.unset_status(GlobalStatus.Status.UPDATES)
                 _LOGGER.debug('ConnectionRefusedError in SSE connection refused. Will try again', exc_info=ex)
             except ConnectionError as ex:
-                self.status &= self.HomeConnectStatus.NOUPDATES
+                #self.status &= self.HomeConnectStatus.NOUPDATES
+                GlobalStatus.unset_status(GlobalStatus.Status.UPDATES)
                 error_code = parse_sse_error(ex.args[0])
                 if error_code == 429:
                     backoff *= 2
@@ -285,7 +282,8 @@ class HomeConnect(DataClassJsonMixin):
                 # it is expected that the connection will time out every hour
                 _LOGGER.debug("The SSE connection timeed-out, will renew and retry")
             except Exception as ex:
-                self.status &= self.HomeConnectStatus.NOUPDATES
+                #self.status &= self.HomeConnectStatus.NOUPDATES
+                GlobalStatus.unset_status(GlobalStatus.Status.UPDATES)
                 _LOGGER.debug('Exception in SSE event stream. Will wait for %d seconds and retry ', backoff, exc_info=ex)
                 await asyncio.sleep(backoff)
                 backoff *= 2
@@ -296,7 +294,8 @@ class HomeConnect(DataClassJsonMixin):
                     await event_source.close()
                     event_source = None
 
-        self.status &= self.HomeConnectStatus.NOUPDATES
+        #self.status &= self.HomeConnectStatus.NOUPDATES
+        GlobalStatus.unset_status(GlobalStatus.Status.UPDATES)
         _LOGGER.debug("Exiting SSE event stream")
 
 
@@ -305,7 +304,12 @@ class HomeConnect(DataClassJsonMixin):
         haid = event.last_event_id
         if event.type == 'KEEP-ALIVE':
             self._last_update = datetime.now()
-        elif event.type == 'PAIRED':
+            return
+        if haid not in self.appliances:
+            # handle cases where the appliance wasn't loaded before
+            _LOGGER.debug("Unknown haId '%s' reloading HomeConnected from the API", haid)
+            await self.async_load_data()
+        if event.type == 'PAIRED':
             self.appliances[haid] = await Appliance.async_create(self, haId=haid)
             await self._callbacks.async_broadcast_event(self.appliances[haid],  Events.PAIRED)
         elif event.type == 'DEPAIRED':
@@ -327,12 +331,15 @@ class HomeConnect(DataClassJsonMixin):
             # Type is NOTIFY or EVENT
             data = json.loads(event.data)
             haid = data['haId']
+            if haid not in self.appliances:
+                _LOGGER.debug("Unknown haId '%s' reloading HomeConnected from the API", haid)
+                await self.async_load_data()
             if 'items' in data:
                 for item in data['items']:
                     # haid = self._get_haId_from_event(item) if 'uri' in item else haid
                     if haid in self.appliances:
                         appliance = self.appliances[haid]
-                        await appliance.async_update_data(item['key'], item['value'])
+                        await appliance.async_update_data(item)
 
 
     # def _get_haId_from_event(self, event:dict):
